
dcoxs<-function(x,y,gse1,gse2,method='pearson'){
        overlap.id<-intersect(gse1,gse2)
        if(length(overlap.id)==0) return(dcoxs.mi(x,y,gse1,gse2,method=method))
        if(ir(gse1,gse2)) return(dcoxs.ir(x,y,gse1,gse2,method=method)) else  return(dcoxs.inter(x,y,gse1,gse2,method=method))
}


dcoxs.mi<-function(x,y,gse1,gse2,method='pearson'){
     
        n1<-ncol(x)*(ncol(x)-1)/2
        n2<-ncol(y)*(ncol(y)-1)/2	
	gs.elements<-list()
	 
	con1<-ias(x[gse1,],x[gse2,],method=method)
	con2<-ias(y[gse1,],y[gse2,],method=method)
	diff.ias<-compcorr(n1,con1[[1]],n2,con2[[1]])

	gs.elements[[1]]<-gse1
	gs.elements[[2]]<-gse2

        list(con1[[1]],con1[[2]],con2[[1]],con2[[2]],diff.ias[[1]],diff.ias[[2]],gs.elements)
}


dcoxs.ir<-function(x,y,gse1,gse2,method='pearson'){

	 n1<-ncol(x)*(ncol(x)-1)/2
         n2<-ncol(y)*(ncol(y)-1)/2

	 overlap.id<-intersect(gse1,gse2)
         union.id<-union(gse1,gse2)
         set.diff.id<-setdiff(union.id,overlap.id)
	
	 gs.elements<-list()
	 gs.elements[[1]]<-set.diff.id
	 gs.elements[[2]]<-overlap.id
	
	 if(length(set.diff.id)<2) return(rep(NA,6))
         
         con1<-ias(x[set.diff.id,],x[overlap.id,],method=method)
	 con2<-ias(y[set.diff.id,],y[overlap.id,],method=method)
	 diff.ias<-compcorr(n1,con1[[1]],n2,con2[[1]])

         list(con1[1],con1[2],con2[1],con2[2],diff.ias[[1]],diff.ias[[2]],gs.elements)
}


dcoxs.inter<-function(x,y,gse1,gse2,method='pearson'){

         n1<-ncol(x)*(ncol(x)-1)/2
         n2<-ncol(y)*(ncol(y)-1)/2

	 overlap.id<-intersect(gse1,gse2)
         #union.id<-union(gse1,gse2)
        	        
         diff.gse1.id<-setdiff(gse1,overlap.id)
         diff.gse2.id<-setdiff(gse2,overlap.id)
	 
	 gs.elements1<-list()
	 gs.elements1[[1]]<-diff.gse1.id
	 gs.elements1[[2]]<-gse2
 
         gs.elements2<-list()
	 gs.elements2[[1]]<-gse1
	 gs.elements2[[2]]<-diff.gse2.id

         if(length(diff.gse1.id)<2) return(rep(NA,6))
         if(length(diff.gse2.id)<2) return(rep(NA,6))
      
	 con11<-ias(x[diff.gse1.id,],x[gse2,],method=method)
	 con12<-ias(y[diff.gse1.id,],y[gse2,],method=method)
	 con21<-ias(x[gse1,],x[diff.gse2.id,],method=method)
	 con22<-ias(y[gse1,],y[diff.gse2.id,],method=method) 
	 
	 diff.ias1<-compcorr(n1,con11[1],n2,con12[1])
	 diff.ias2<-compcorr(n1,con21[1],n2,con22[1])

	 if(is.na(diff.ias1[[1]]) || is.na(diff.ias2[[1]])) return(rep(NA,6))
	 
	 if(abs(diff.ias1[[1]])>abs(diff.ias2[[1]])){
	    return(list(con11[1],con11[2],con12[1],con12[2],diff.ias1[[1]],diff.ias1[[2]],gs.elements1))
	 }else{
	    return(list(con21[1],con21[2],con22[1],con22[2],diff.ias2[[1]],diff.ias2[[2]],gs.elements2))
	 }
}


ir<-function(a,b) if(length(union(a,b))==length(a)  || length(union(a,b))==length(b)) return(TRUE) else return(FALSE)


ias<-function(x,y,method='pearson'){
      
         if(ncol(x)!=ncol(y)) stop('The sample numbers of two expression profiles are different!')

         rx<-RRE.matrix(x)
         ry<-RRE.matrix(y)

         rx<-rx[upper.tri(rx)]
         ry<-ry[upper.tri(ry)]
	
         if(length(which(!is.finite(rx)))>0 || length(which(!is.finite(ry)))>0) return(c(NA,NA))
	 if(sd(rx)==0 || sd(ry)==0) return(c(NA,NA))

	 tmp<-cor.test(rx,ry,method=method)
	 if(is.na(tmp[[4]])) return(c(NA,NA))
         c(tmp[[4]],tmp[[3]])
}

compcorr <- function(n1, r1, n2, r2){ 
	 zf1 <- 0.5*log((1 + r1)/(1 - r1))
	 zf2 <- 0.5*log((1 + r2)/(1 - r2))
         dz <- (zf1 - zf2)/sqrt(1/(n1 - 3) + (1/(n2 - 3))) 
         pv <- 2*(1 - pnorm(abs(dz)))
	 return(list(diff=dz, pval=pv))
} 

#  implementation of rule-of-thumb band width selection by Scott (92)   

md.bw.estimation<-function(x){
         sigma<-apply(x,MARGIN=1,FUN=sd)
         h<-sigma*ncol(x)^(-1/(nrow(x)+4))
         h
}


#  multivariate product kernel     
#  x is an instance, y is a total set of samples, bw is a list of bandwidth 
#  using gaussian kernel
#  row of y means gene, column of y means sample 

md.kernel<-function(x,y){
          kk<-c()
          h<-md.bw.estimation(y)
      
          for(i in 1:ncol(y)){
             k<-c()
             for(j in 1:length(x)){
	         k<-c(k,(2*pi)^(-0.5)*exp(-(x[j]-y[j,i])^2/(2*(h[j]^2))))
	         k<-prod(k)
	     }
	     kk<-c(kk,k)
          }
          1/(ncol(y)*prod(h))*sum(kk)
}



# kernel density estimation of a sample set

md.kernel.set<- function(x){
           re<-c()
	   for(i in 1:ncol(x)) re<-c(md.kernel(x[,i],x[,-i]),re)
           re
}
 	    

# Renyi's quadratic relative entropy
# Sample-wise entropy matrix 
    
RRE.matrix<-function(x){
            k<-ncol(x)
            re<-matrix(nrow=k,ncol=k)
	    tm<-md.kernel.set(x)                
	    for(i in 1:k){
	       for(j in 1:k){re[i,j]<-log((tm[i]+.Machine$double.eps)/(tm[j]+.Machine$double.eps))}
	    }
	    re
}

